import { isAsync, waitFor } from "../../gensync-utils/async.js";
import path from "path";
import { pathToFileURL } from "url";
import { createRequire } from "module";
import semver from "semver";
import buildDebug from "debug";
import { endHiddenCallStack } from "../../errors/rewrite-stack-trace.js";
import ConfigError from "../../errors/config-error.js";
import { transformFileSync } from "../../transform-file.js";
const debug = buildDebug("babel:config:loading:files:module-types");
const require = createRequire(import.meta.url);
;
export const supportsESM = semver.satisfies(process.versions.node, "^12.17 || >=13.2");
const LOADING_CJS_FILES = new Set();
function loadCjsDefault(filepath) {
  if (LOADING_CJS_FILES.has(filepath)) {
    debug("Auto-ignoring usage of config %o.", filepath);
    return {};
  }
  let module;
  try {
    LOADING_CJS_FILES.add(filepath);
    module = endHiddenCallStack(require)(filepath);
  } finally {
    LOADING_CJS_FILES.delete(filepath);
  }
  {
    return module != null && (module.__esModule || module[Symbol.toStringTag] === "Module") ? module.default : module;
  }
}
const loadMjsFromPath = endHiddenCallStack(async function loadMjsFromPath(filepath) {
  const url = pathToFileURL(filepath).toString() + "?import";
  {
    return await import(url);
  }
});
const SUPPORTED_EXTENSIONS = new Set([".js", ".mjs", ".cjs", ".cts"]);
const asyncModules = new Set();
export default function* loadCodeDefault(filepath, loader, esmError, tlaError) {
  let async;
  let ext = path.extname(filepath);
  if (!SUPPORTED_EXTENSIONS.has(ext)) ext = ".js";
  const pattern = `${loader} ${ext}`;
  switch (pattern) {
    case "require .cjs":
    case "auto .cjs":
      {
        return loadCjsDefault(filepath);
      }
    case "require .cts":
    case "auto .cts":
      return loadCtsDefault(filepath);
    case "auto .js":
    case "require .js":
    case "require .mjs":
      try {
        {
          return loadCjsDefault(filepath);
        }
      } catch (e) {
        if (e.code === "ERR_REQUIRE_ASYNC_MODULE" || e.code === "ERR_REQUIRE_CYCLE_MODULE" && asyncModules.has(filepath)) {
          asyncModules.add(filepath);
          if (!(async ??= yield* isAsync())) {
            throw new ConfigError(tlaError, filepath);
          }
        } else if (e.code === "ERR_REQUIRE_ESM") {} else {
          throw e;
        }
      }
    case "auto .mjs":
      if (async ??= yield* isAsync()) {
        return (yield* waitFor(loadMjsFromPath(filepath))).default;
      }
      throw new ConfigError(esmError, filepath);
    default:
      throw new Error("Internal Babel error: unreachable code.");
  }
}
function loadCtsDefault(filepath) {
  const ext = ".cts";
  const hasTsSupport = !!(require.extensions[".ts"] || require.extensions[".cts"] || require.extensions[".mts"]);
  let handler;
  if (!hasTsSupport) {
    const opts = {
      babelrc: false,
      configFile: false,
      sourceType: "unambiguous",
      sourceMaps: "inline",
      sourceFileName: path.basename(filepath),
      presets: [[getTSPreset(filepath), Object.assign({
        onlyRemoveTypeImports: true,
        optimizeConstEnums: true
      }, {})]]
    };
    handler = function (m, filename) {
      if (handler && filename.endsWith(ext)) {
        try {
          return m._compile(transformFileSync(filename, Object.assign({}, opts, {
            filename
          })).code, filename);
        } catch (error) {
          if (!hasTsSupport) {
            const packageJson = require("@babel/preset-typescript/package.json");
            if (semver.lt(packageJson.version, "7.21.4")) {
              console.error("`.cts` configuration file failed to load, please try to update `@babel/preset-typescript`.");
            }
          }
          throw error;
        }
      }
      return require.extensions[".js"](m, filename);
    };
    require.extensions[ext] = handler;
  }
  try {
    return loadCjsDefault(filepath);
  } finally {
    if (!hasTsSupport) {
      if (require.extensions[ext] === handler) delete require.extensions[ext];
      handler = undefined;
    }
  }
}
function getTSPreset(filepath) {
  try {
    return require("@babel/preset-typescript");
  } catch (error) {
    if (error.code !== "MODULE_NOT_FOUND") throw error;
    let message = "You appear to be using a .cts file as Babel configuration, but the `@babel/preset-typescript` package was not found: please install it!";
    ;
    throw new ConfigError(message, filepath);
  }
}

//# sourceMappingURL=module-types.js.map
